@@ -38,7 +38,6 @@ class Aggregator:
         self.log = Logger("aggregator")
         self.data_dir = os.path.expanduser(os.environ.get('DATA_DIR','data'))
         self.outputs_dir = os.path.join(os.getcwd(), 'outputs')
-        print(self.outputs_dir)
         if not os.path.isdir(self.outputs_dir):
             os.makedirs(self.outputs_dir)
         self.config_file = os.path.join(self.data_dir, os.environ.get('CONFIG_FILE', 'config.toml'))
@@ -86,9 +85,10 @@ class Aggregator:
         self.max_daily_ghi = None
         self.min_daily_temp = None
         self.prev_load = None
-        self.ts_data = self._import_ts_data() # Temp: degC, RH: %, Pressure: mbar, GHI: W/m2
+        self.dt = int(self.config['agg']['subhourly_steps'])
         self._set_dt()
-
+        self.ts_data = self._import_ts_data() # Temp: degC, RH: %, Pressure: mbar, GHI: W/m2
+        
         self.spp_data = self._import_spp_data() # SPP: $/kWh
         self.tou_data = self._build_tou_price() # TOU: $/kWh
         self.all_data = self.join_data()
@@ -131,8 +131,9 @@ class Aggregator:
         :return:
         """
         try: 
-            if not self.str_start_dt and not self.str_end_dt:
+            if not self.str_start_dt:
                 self.str_start_dt = self.config["simulation"]["start_datetime"]
+            if not self.str_end_dt:
                 self.str_end_dt = self.config["simulation"]["end_datetime"]
             self.start_dt = datetime.strptime(self.str_start_dt, '%Y-%m-%d %H')
             self.end_dt = datetime.strptime(self.str_end_dt, '%Y-%m-%d %H')
@@ -157,30 +158,40 @@ class Aggregator:
             sys.exit(1)
 
         df = pd.read_csv(self.ts_data_file, skiprows=2)
-        self.dt = int(self.config['agg']['subhourly_steps'])
+
+        
         self.dt_interval = 60 // self.dt
-        reps = [np.ceil(self.dt/2) if val==0 else np.floor(self.dt/2) for val in df.Minute]
-        df = df.loc[np.repeat(df.index.values, reps)]
-        interval_minutes = self.dt_interval * np.arange(self.dt)
-        n_intervals = len(df.index) // self.dt
-        x = np.tile(interval_minutes, n_intervals)
-        df.Minute = x
-        df = df.astype(str)
-        df['ts'] = df[["Year", "Month", "Day", "Hour", "Minute"]].apply(lambda x: ' '.join(x), axis=1)
+
+        # read in original data
+        df["ts"] = pd.to_datetime(df[['Year','Month','Day','Hour','Minute']])
         df = df.rename(columns={"Temperature": "OAT"})
-        df["ts"] = df["ts"].apply(lambda x: datetime.strptime(x, '%Y %m %d %H %M'))
-        df = df.filter(["ts", "GHI", "OAT"])
-        df[["GHI", "OAT"]] = df[["GHI", "OAT"]].astype(int)
+        df.set_index('ts', inplace=True)
+
+        # create interpolated index
+        df_interp = pd.DataFrame(index=pd.date_range(start=df.index.min(),end=df.index.max(),
+                                                          freq=f'{self.dt_interval}T'))
+
+        # create merged data and interpolate to fill missing points
+        df = pd.concat([df, df_interp]).sort_index().interpolate('linear')
+        df = df[~df.index.duplicated(keep='first')]
+
+        # filter to only interpolated data (exclude non-control datetimes)
+        df.filter(df_interp.index, axis=0)
+        df.bfill(inplace=True)
+
         self.oat = df['OAT'].to_numpy()
         self.ghi = df['GHI'].to_numpy()
-        df = df.set_index('ts')
-
-        day_of_year = 0
-        self.thermal_trend = self.oat[4 * self.dt] - self.oat[0]
-        self.max_daily_temp = max(self.oat[day_of_year*(self.dt*24):(day_of_year+1)*(self.dt*24)])
-        self.min_daily_temp = min(self.oat[day_of_year*(self.dt*24):(day_of_year+1)*(self.dt*24)])
-        self.max_daily_ghi = max(self.ghi[day_of_year*(self.dt*24):(day_of_year+1)*(self.dt*24)])
 
+        self.start_index = df.index.get_loc(self.start_dt, method='nearest') # the index of the start datetime w/r/t the entire year
+        self.end_index = df.index.get_loc(self.end_dt, method='nearest')
+        idx = df.index[self.start_index]
+        idx_h = df.index[self.start_index + 4 * self.dt]
+        idx_end = df.index[self.end_index]
+        self.thermal_trend = df.loc[idx_h, 'OAT'] - df.loc[idx, 'OAT']
+        self.max_daily_temp = max(df.loc[df.index.date >= self.start_dt.date()]["OAT"])
+        self.min_daily_temp = min(df.loc[df.index.date >= self.start_dt.date()]["OAT"])
+        self.max_daily_ghi = max(df.loc[df.index.date >= self.start_dt.date()]["GHI"])
+        
         return df
 
     def _import_spp_data(self):
@@ -706,7 +717,8 @@ class Aggregator:
 
         self.redis_client.set("start_hour_index", self.start_hour_index)
         self.redis_client.hset("current_values", "timestep", self.timestep)
-
+        self.redis_client.hset("current_values", "start_index", self.start_index)
+        self.redis_client.hset("current_values", "end_index", self.end_index)
         self.reward_price = np.zeros(self.config['agg']['rl']['action_horizon'] * self.dt)
         self.redis_client.rpush("reward_price", *self.reward_price.tolist())
 
@@ -717,6 +729,7 @@ class Aggregator:
         as the data in self.all_data, which is 8760 for default config file.
         :return: None
         """
+        self.all_data.bfill(inplace=True)
         for c in self.all_data.columns.to_list():
             self.redis_client.delete(c)
             self.redis_client.rpush(c, *self.all_data[c].values.tolist())
@@ -783,7 +796,7 @@ class Aggregator:
         pool = ProcessPool(nodes=self.config['simulation']['n_nodes']) # open a pool of nodes
         results = pool.map(manage_home, self.mpc_players)
 
-        self.timestep += 1
+        self.timestep = (self.timestep + 1) % self.num_timesteps
 
     def collect_data(self):
         """
@@ -797,7 +810,6 @@ class Aggregator:
         for home in self.all_homes:
             if self.check_type == 'all' or home["type"] == self.check_type:
                 vals = self.redis_client.hgetall(home["name"])
-                print(home["name"])
                 for k, v in vals.items():
                     opt_keys = ["p_grid_opt", "forecast_p_grid_opt", "p_load_opt", "temp_in_opt", "temp_wh_opt", "hvac_cool_on_opt", "hvac_heat_on_opt", "wh_heat_on_opt", "cost_opt", "waterdraws", "correct_solve", "t_in_max", "t_in_min"]
                     if 'pv' in home["type"]:
